@@ -9,16 +9,19 @@
 #include "utils.h"
 #include "calendarsupport_debug.h"
 #include "kcalprefs.h"
+#include <kholidays_version.h>
 
-#include <AkonadiCore/AgentInstance>
-#include <AkonadiCore/AgentManager>
-#include <AkonadiCore/EntityDisplayAttribute>
-#include <AkonadiCore/EntityTreeModel>
+#include <Akonadi/AgentInstance>
+#include <Akonadi/AgentManager>
+#include <Akonadi/CalendarUtils>
+#include <Akonadi/CollectionUtils>
+#include <Akonadi/EntityDisplayAttribute>
+#include <Akonadi/EntityTreeModel>
 
-#include <AkonadiWidgets/CollectionDialog>
+#include <Akonadi/CollectionDialog>
 
-#include <Akonadi/Calendar/BlockAlarmsAttribute>
-#include <Akonadi/Calendar/ETMCalendar>
+#include <Akonadi/BlockAlarmsAttribute>
+#include <Akonadi/ETMCalendar>
 
 #include <KHolidays/HolidayRegion>
 
@@ -41,81 +44,31 @@
 #include <QStyle>
 #include <QUrlQuery>
 
-KCalendarCore::Incidence::Ptr CalendarSupport::incidence(const Akonadi::Item &item)
-{
-    // relying on exception for performance reasons
-    try {
-        return item.payload<KCalendarCore::Incidence::Ptr>();
-    } catch (const Akonadi::PayloadException &) {
-        return KCalendarCore::Incidence::Ptr();
-    }
-}
-
-KCalendarCore::Event::Ptr CalendarSupport::event(const Akonadi::Item &item)
-{
-    // relying on exception for performance reasons
-    try {
-        auto incidence = item.payload<KCalendarCore::Incidence::Ptr>();
-        if (hasEvent(incidence)) {
-            return item.payload<KCalendarCore::Event::Ptr>();
-        }
-    } catch (const Akonadi::PayloadException &) {
-        return KCalendarCore::Event::Ptr();
-    }
-    return KCalendarCore::Event::Ptr();
-}
-
 KCalendarCore::Event::Ptr CalendarSupport::event(const KCalendarCore::Incidence::Ptr &incidence)
 {
     if (hasEvent(incidence)) {
         return incidence.staticCast<KCalendarCore::Event>();
     }
-    return KCalendarCore::Event::Ptr();
+    return {};
 }
 
 KCalendarCore::Incidence::List CalendarSupport::incidencesFromItems(const Akonadi::Item::List &items)
 {
     KCalendarCore::Incidence::List incidences;
     for (const Akonadi::Item &item : items) {
-        if (const KCalendarCore::Incidence::Ptr e = CalendarSupport::incidence(item)) {
+        if (const KCalendarCore::Incidence::Ptr e = Akonadi::CalendarUtils::incidence(item)) {
             incidences.push_back(e);
         }
     }
     return incidences;
 }
 
-KCalendarCore::Todo::Ptr CalendarSupport::todo(const Akonadi::Item &item)
-{
-    try {
-        auto incidence = item.payload<KCalendarCore::Incidence::Ptr>();
-        if (hasTodo(incidence)) {
-            return item.payload<KCalendarCore::Todo::Ptr>();
-        }
-    } catch (const Akonadi::PayloadException &) {
-        return KCalendarCore::Todo::Ptr();
-    }
-    return KCalendarCore::Todo::Ptr();
-}
-
 KCalendarCore::Todo::Ptr CalendarSupport::todo(const KCalendarCore::Incidence::Ptr &incidence)
 {
     if (hasTodo(incidence)) {
         return incidence.staticCast<KCalendarCore::Todo>();
     }
-    return KCalendarCore::Todo::Ptr();
-}
-
-KCalendarCore::Journal::Ptr CalendarSupport::journal(const Akonadi::Item &item)
-{
-    try {
-        auto incidence = item.payload<KCalendarCore::Incidence::Ptr>();
-        if (hasJournal(incidence)) {
-            return item.payload<KCalendarCore::Journal::Ptr>();
-        }
-    } catch (const Akonadi::PayloadException &) {
-        return KCalendarCore::Journal::Ptr();
-    }
-    return KCalendarCore::Journal::Ptr();
+    return {};
 }
 
 KCalendarCore::Journal::Ptr CalendarSupport::journal(const KCalendarCore::Incidence::Ptr &incidence)
@@ -123,7 +76,7 @@ KCalendarCore::Journal::Ptr CalendarSupport::journal(const KCalendarCore::Incide
     if (hasJournal(incidence)) {
         return incidence.staticCast<KCalendarCore::Journal>();
     }
-    return KCalendarCore::Journal::Ptr();
+    return {};
 }
 
 bool CalendarSupport::hasIncidence(const Akonadi::Item &item)
@@ -172,7 +125,7 @@ QMimeData *CalendarSupport::createMimeData(const Akonadi::Item::List &items)
     QList<QUrl> urls;
     int incidencesFound = 0;
     for (const Akonadi::Item &item : items) {
-        const KCalendarCore::Incidence::Ptr incidence(CalendarSupport::incidence(item));
+        const KCalendarCore::Incidence::Ptr incidence(Akonadi::CalendarUtils::incidence(item));
         if (!incidence) {
             continue;
         }
@@ -197,33 +150,24 @@ QMimeData *CalendarSupport::createMimeData(const Akonadi::Item::List &items)
     }
 }
 
-QMimeData *CalendarSupport::createMimeData(const Akonadi::Item &item)
-{
-    return createMimeData(Akonadi::Item::List() << item);
-}
-
 #ifndef QT_NO_DRAGANDDROP
-QDrag *CalendarSupport::createDrag(const Akonadi::Item &item, QWidget *parent)
+QDrag *CalendarSupport::createDrag(const Akonadi::Item &item, QObject *parent)
 {
     return createDrag(Akonadi::Item::List() << item, parent);
 }
 
 #endif
 
-static QByteArray findMostCommonType(const Akonadi::Item::List &items)
+static KCalendarCore::IncidenceBase::IncidenceType findMostCommonType(const Akonadi::Item::List &items)
 {
-    QByteArray prev;
-    if (items.isEmpty()) {
-        return "Incidence";
-    }
-
+    KCalendarCore::IncidenceBase::IncidenceType prev = KCalendarCore::IncidenceBase::TypeUnknown;
     for (const Akonadi::Item &item : items) {
         if (!CalendarSupport::hasIncidence(item)) {
             continue;
         }
-        const QByteArray type = CalendarSupport::incidence(item)->typeStr();
-        if (!prev.isEmpty() && type != prev) {
-            return "Incidence";
+        const auto type = Akonadi::CalendarUtils::incidence(item)->type();
+        if (prev != KCalendarCore::IncidenceBase::TypeUnknown && type != prev) {
+            return KCalendarCore::IncidenceBase::TypeUnknown;
         }
         prev = type;
     }
@@ -231,15 +175,15 @@ static QByteArray findMostCommonType(const Akonadi::Item::List &items)
 }
 
 #ifndef QT_NO_DRAGANDDROP
-QDrag *CalendarSupport::createDrag(const Akonadi::Item::List &items, QWidget *parent)
+QDrag *CalendarSupport::createDrag(const Akonadi::Item::List &items, QObject *parent)
 {
     std::unique_ptr<QDrag> drag(new QDrag(parent));
     drag->setMimeData(CalendarSupport::createMimeData(items));
 
-    const QByteArray common = findMostCommonType(items);
-    if (common == "Event") {
+    const auto common = findMostCommonType(items);
+    if (common == KCalendarCore::IncidenceBase::TypeEvent) {
         drag->setPixmap(QIcon::fromTheme(QStringLiteral("view-calendar-day")).pixmap(qApp->style()->pixelMetric(QStyle::PM_ToolBarIconSize)));
-    } else if (common == "Todo") {
+    } else if (common == KCalendarCore::IncidenceBase::TypeTodo) {
         drag->setPixmap(QIcon::fromTheme(QStringLiteral("view-calendar-tasks")).pixmap(qApp->style()->pixelMetric(QStyle::PM_ToolBarIconSize)));
     }
 
@@ -251,7 +195,7 @@ QDrag *CalendarSupport::createDrag(const Akonadi::Item::List &items, QWidget *pa
 static bool itemMatches(const Akonadi::Item &item, const KCalendarCore::CalFilter *filter)
 {
     assert(filter);
-    KCalendarCore::Incidence::Ptr inc = CalendarSupport::incidence(item);
+    KCalendarCore::Incidence::Ptr inc = Akonadi::CalendarUtils::incidence(item);
     if (!inc) {
         return false;
     }
@@ -418,7 +362,7 @@ Akonadi::Collection::List CalendarSupport::collectionsFromModel(const QAbstractI
     int row = start;
     QModelIndex i = model->index(row, 0, parentIndex);
     while (row <= endRow) {
-        const Akonadi::Collection collection = collectionFromIndex(i);
+        const Akonadi::Collection collection = Akonadi::CollectionUtils::fromIndex(i);
         if (collection.isValid()) {
             collections << collection;
             QModelIndex childIndex = model->index(0, 0, i);
@@ -454,11 +398,6 @@ Akonadi::Item::List CalendarSupport::itemsFromModel(const QAbstractItemModel *mo
     return items;
 }
 
-Akonadi::Collection CalendarSupport::collectionFromIndex(const QModelIndex &index)
-{
-    return index.data(Akonadi::EntityTreeModel::CollectionRole).value<Akonadi::Collection>();
-}
-
 Akonadi::Collection::Id CalendarSupport::collectionIdFromIndex(const QModelIndex &index)
 {
     return index.data(Akonadi::EntityTreeModel::CollectionIdRole).value<Akonadi::Collection::Id>();
@@ -469,140 +408,11 @@ Akonadi::Collection::List CalendarSupport::collectionsFromIndexes(const QModelIn
     Akonadi::Collection::List l;
     l.reserve(indexes.count());
     for (const QModelIndex &idx : indexes) {
-        l.push_back(collectionFromIndex(idx));
+        l.push_back(Akonadi::CollectionUtils::fromIndex(idx));
     }
     return l;
 }
 
-QString CalendarSupport::displayName(Akonadi::ETMCalendar *calendar, const Akonadi::Collection &c)
-{
-    Akonadi::Collection fullCollection;
-    if (calendar && calendar->collection(c.id()).isValid()) {
-        fullCollection = calendar->collection(c.id());
-    } else {
-        fullCollection = c;
-    }
-
-    QString cName = fullCollection.name();
-    const QString resourceName = fullCollection.resource();
-
-    // Kolab Groupware
-    if (resourceName.contains(QLatin1String("kolab"))) {
-        QString typeStr = cName; // contents type: "Calendar", "Tasks", etc
-        QString ownerStr; // folder owner: "fred", "ethel", etc
-        QString nameStr; // folder name: "Public", "Test", etc
-        if (calendar) {
-            Akonadi::Collection p = c.parentCollection();
-            while (p != Akonadi::Collection::root()) {
-                Akonadi::Collection tCol = calendar->collection(p.id());
-                const QString tName = tCol.name();
-                if (tName.startsWith(QLatin1String("shared.cal"), Qt::CaseInsensitive)) {
-                    ownerStr = QStringLiteral("Shared");
-                    nameStr = cName;
-                    typeStr = i18n("Calendar");
-                    break;
-                } else if (tName.startsWith(QLatin1String("shared.tasks"), Qt::CaseInsensitive)
-                           || tName.startsWith(QLatin1String("shared.todo"), Qt::CaseInsensitive)) {
-                    ownerStr = QStringLiteral("Shared");
-                    nameStr = cName;
-                    typeStr = i18n("Tasks");
-                    break;
-                } else if (tName.startsWith(QLatin1String("shared.journal"), Qt::CaseInsensitive)) {
-                    ownerStr = QStringLiteral("Shared");
-                    nameStr = cName;
-                    typeStr = i18n("Journal");
-                    break;
-                } else if (tName.startsWith(QLatin1String("shared.notes"), Qt::CaseInsensitive)) {
-                    ownerStr = QStringLiteral("Shared");
-                    nameStr = cName;
-                    typeStr = i18n("Notes");
-                    break;
-                } else if (tName != i18n("Calendar") && tName != i18n("Tasks") && tName != i18n("Journal") && tName != i18n("Notes")) {
-                    ownerStr = tName;
-                    break;
-                } else {
-                    nameStr = typeStr;
-                    typeStr = tName;
-                }
-                p = p.parentCollection();
-            }
-        }
-
-        if (!ownerStr.isEmpty()) {
-            if (!ownerStr.compare(QLatin1String("INBOX"), Qt::CaseInsensitive)) {
-                return i18nc("%1 is folder contents", "My Kolab %1", typeStr);
-            } else if (!ownerStr.compare(QLatin1String("SHARED"), Qt::CaseInsensitive) || !ownerStr.compare(QLatin1String("CALENDAR"), Qt::CaseInsensitive)
-                       || !ownerStr.compare(QLatin1String("RESOURCES"), Qt::CaseInsensitive)) {
-                return i18nc("%1 is folder name, %2 is folder contents", "Shared Kolab %1 %2", nameStr, typeStr);
-            } else {
-                if (nameStr.isEmpty()) {
-                    return i18nc("%1 is folder owner name, %2 is folder contents", "%1's Kolab %2", ownerStr, typeStr);
-                } else {
-                    return i18nc("%1 is folder owner name, %2 is folder name, %3 is folder contents", "%1's %2 Kolab %3", ownerStr, nameStr, typeStr);
-                }
-            }
-        } else {
-            return i18nc("%1 is folder contents", "Kolab %1", typeStr);
-        }
-    } // end kolab section
-
-    // Dav Groupware
-    if (resourceName.contains(QLatin1String("davgroupware"))) {
-        const QString resourceDisplayName = Akonadi::AgentManager::self()->instance(resourceName).name();
-        return i18nc("%1 is the folder name", "%1 in %2", fullCollection.displayName(), resourceDisplayName);
-    } // end caldav section
-
-    // Google
-    if (resourceName.contains(QLatin1String("google"))) {
-        QString ownerStr; // folder owner: "user@gmail.com"
-        if (calendar) {
-            Akonadi::Collection p = c.parentCollection();
-            ownerStr = calendar->collection(p.id()).displayName();
-        }
-
-        const QString nameStr = c.displayName(); // folder name: can be anything
-
-        QString typeStr;
-        const QString mimeStr = c.contentMimeTypes().join(QLatin1Char(','));
-        if (mimeStr.contains(QLatin1String(".event"))) {
-            typeStr = i18n("Calendar");
-        } else if (mimeStr.contains(QLatin1String(".todo"))) {
-            typeStr = i18n("Tasks");
-        } else if (mimeStr.contains(QLatin1String(".journal"))) {
-            typeStr = i18n("Journal");
-        } else if (mimeStr.contains(QLatin1String(".note"))) {
-            typeStr = i18n("Notes");
-        } else {
-            typeStr = mimeStr;
-        }
-
-        if (!ownerStr.isEmpty()) {
-            const int atChar = ownerStr.lastIndexOf(QLatin1Char('@'));
-            if (atChar >= 0) {
-                ownerStr.truncate(atChar);
-            }
-            if (nameStr.isEmpty()) {
-                return i18nc("%1 is folder owner name, %2 is folder contents", "%1's Google %2", ownerStr, typeStr);
-            } else {
-                return i18nc("%1 is folder owner name, %2 is folder name", "%1's %2", ownerStr, nameStr);
-            }
-        } else {
-            return i18nc("%1 is folder contents", "Google %1", typeStr);
-        }
-    } // end google section
-
-    // Not groupware so the collection is "mine"
-    const QString dName = fullCollection.displayName();
-
-    if (!dName.isEmpty()) {
-        return fullCollection.name().startsWith(QLatin1String("akonadi_")) ? i18n("My %1", dName) : dName;
-    } else if (!fullCollection.name().isEmpty()) {
-        return fullCollection.name();
-    } else {
-        return i18nc("unknown resource", "Unknown");
-    }
-}
-
 QString CalendarSupport::toolTipString(const Akonadi::Collection &coll, bool richText)
 {
     Q_UNUSED(richText)
@@ -716,7 +526,11 @@ QList<QDate> CalendarSupport::workDays(QDate startDate, QDate endDate)
         for (const QString &regionStr : holidays) {
             KHolidays::HolidayRegion region(regionStr);
             if (region.isValid()) {
-                for (auto const h : region.holidays(startDate, endDate)) {
+#if KHOLIDAYS_VERSION < QT_VERSION_CHECK(5, 95, 0)
+                for (auto const &h : region.holidays(startDate, endDate)) {
+#else
+                for (auto const &h : region.rawHolidaysWithAstroSeasons(startDate, endDate)) {
+#endif
                     if (h.dayType() == KHolidays::Holiday::NonWorkday) {
                         for (int i = 0; i < h.duration(); i++) {
                             result.removeOne(h.observedStartDate().addDays(i));
@@ -739,7 +553,11 @@ QStringList CalendarSupport::holiday(QDate date)
     for (const QString &regionStr : holidays) {
         KHolidays::HolidayRegion region(regionStr);
         if (region.isValid()) {
+#if KHOLIDAYS_VERSION < QT_VERSION_CHECK(5, 95, 0)
             const KHolidays::Holiday::List list = region.holidays(date);
+#else
+            const KHolidays::Holiday::List list = region.rawHolidaysWithAstroSeasons(date);
+#endif
             const int listCount = list.count();
             for (int i = 0; i < listCount; ++i) {
                 // don't add duplicates.
@@ -773,7 +591,8 @@ QStringList CalendarSupport::holiday(QDate date)
 
 QStringList CalendarSupport::categories(const KCalendarCore::Incidence::List &incidences)
 {
-    QStringList cats, thisCats;
+    QStringList cats;
+    QStringList thisCats;
     // @TODO: For now just iterate over all incidences. In the future,
     // the list of categories should be built when reading the file.
     for (const KCalendarCore::Incidence::Ptr &incidence : incidences) {
